package org.maachang.dbm.engine ;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * MDbm定数初期化処理.
 * 
 * @version 2008/05/24
 * @author masahito suzuki
 * @since MaachangDBM 1.0E
 */
public class MDbmEnv {
    
    /**
     * Hash数.
     */
    public static final int HASH ;
    
    /**
     * １つのKeyで管理できる個数.
     */
    public static final int KEY ;
    
    /**
     * Key最大保持数.
     */
    public static final int MAX_KEY_LENGTH ;
    
    /**
     * １セクター長.
     */
    public static final int SECTOR ;
    
    /**
     * １セクタファイルで扱える最大数.
     */
    public static final int ONE_SECTOR_BY_SECTOR_SIZE ;
    
    
    
    
    /**
     * 初期化処理.
     */
    static {
        
        // ログ出力ファイル条件.
        Log logOut = LogFactory.getLog( MDbmEnv.class ) ;
        
        // バージョンを表示.
        if( logOut.isDebugEnabled() ) {
            logOut.debug( "## MDbm version:" + MDbmDefine.VERSION ) ;
        }
        
        // データ受け取り一時条件.
        int n ;
        
        //////////////////////////////////////////////////////////////
        // Hash設定.
        //////////////////////////////////////////////////////////////
        n = MDbmDefine.DEF_HASH ;
        if( System.getProperty( MDbmDefine.HASH_PROPERTY ) != null ) {
            try {
                n = Integer.parseInt( System.getProperty( MDbmDefine.HASH_PROPERTY ) ) ;
                if( n >= MDbmDefine.MAX_HASH ) {
                    n = MDbmDefine.MAX_HASH ;
                }
                else if( n <= MDbmDefine.MIN_HASH ) {
                    n = MDbmDefine.MIN_HASH ;
                }
            } catch( Exception e ) {
                n = MDbmDefine.DEF_HASH ;
            }
        }
        int hashSize = 0x000fffff ;
        switch( n ) {
            case 16 : hashSize = 0x0000ffff ; break ;
            case 17 : hashSize = 0x0001ffff ; break ;
            case 18 : hashSize = 0x0003ffff ; break ;
            case 19 : hashSize = 0x0007ffff ; break ;
            case 20 : hashSize = 0x000fffff ; break ;
            case 21 : hashSize = 0x001fffff ; break ;
            case 22 : hashSize = 0x003fffff ; break ;
            case 23 : hashSize = 0x007fffff ; break ;
            case 24 : hashSize = 0x00ffffff ; break ;
            case 25 : hashSize = 0x01ffffff ; break ;
            case 26 : hashSize = 0x03ffffff ; break ;
            case 27 : hashSize = 0x07ffffff ; break ;
            case 28 : hashSize = 0x0fffffff ; break ;
            case 29 : hashSize = 0x1fffffff ; break ;
            case 30 : hashSize = 0x3fffffff ; break ;
            case 31 : hashSize = 0x7fffffff ; break ;
        }
        HASH = hashSize + 1 ;
        if( logOut.isDebugEnabled() ) {
            logOut.debug( "## MDbmEnv.HASH:" + ( hashSize + 1 ) ) ;
        }
        
        //////////////////////////////////////////////////////////////
        // Key設定.
        //////////////////////////////////////////////////////////////
        
        // key長を設定.
        n = MDbmDefine.DEF_KEY ;
        if( System.getProperty( MDbmDefine.KEY_PROPERTY ) != null ) {
            try {
                n = Integer.parseInt( System.getProperty( MDbmDefine.KEY_PROPERTY ) ) ;
                if( n >= MDbmDefine.MAX_KEY ) {
                    n = MDbmDefine.MAX_KEY ;
                }
                else if( n <= MDbmDefine.MIN_KEY ) {
                    n = MDbmDefine.MIN_KEY ;
                }
            } catch( Exception e ) {
                n = MDbmDefine.DEF_KEY ;
            }
        }
        n = ( int )( n / MDbmDefine.MIN_KEY ) * MDbmDefine.MIN_KEY ;
        if( n <= MDbmDefine.MIN_KEY ) {
            n = MDbmDefine.MIN_KEY ;
        }
        KEY = n ;
        if( logOut.isDebugEnabled() ) {
            logOut.debug( "## MDbmEnv.KEY:" + n ) ;
        }
        
        // 最大格納件数を設定.
        n = MDbmDefine.DEF_LENGTH ;
        if( System.getProperty( MDbmDefine.LENGTH_PROPERTY ) != null ) {
            try {
                n = Integer.parseInt( System.getProperty( MDbmDefine.LENGTH_PROPERTY ) ) ;
                if( n >= MDbmDefine.MAX_LENGTH ) {
                    n = MDbmDefine.MAX_LENGTH ;
                }
                else if( n <= MDbmDefine.MIN_LENGTH ) {
                    n = MDbmDefine.MIN_LENGTH ;
                }
            } catch( Exception e ) {
                n = MDbmDefine.DEF_LENGTH ;
            }
        }
        n = ( int )( n / MDbmDefine.MIN_LENGTH ) * MDbmDefine.MIN_LENGTH ;
        if( n <= MDbmDefine.MIN_LENGTH ) {
            n = MDbmDefine.MIN_LENGTH ;
        }
        MAX_KEY_LENGTH = n ;
        if( logOut.isDebugEnabled() ) {
            logOut.debug( "## MDbmEnv.MAX_KEY_LENGTH:" + n ) ;
        }
        
        //////////////////////////////////////////////////////////////
        // Sector設定.
        //////////////////////////////////////////////////////////////
        
        // セクタ数.
        n = MDbmDefine.DEF_SECTOR ;
        if( System.getProperty( MDbmDefine.SECTOR_PROPERTY ) != null ) {
            try {
                n = Integer.parseInt( System.getProperty( MDbmDefine.SECTOR_PROPERTY ) ) ;
                if( n >= MDbmDefine.MAX_SECTOR ) {
                    n = MDbmDefine.MAX_SECTOR ;
                }
                else if( n <= MDbmDefine.MIN_SECTOR ) {
                    n = MDbmDefine.MIN_SECTOR ;
                }
            } catch( Exception e ) {
                n = MDbmDefine.DEF_SECTOR ;
            }
        }
        n = ( int )( n / MDbmDefine.MIN_SECTOR ) * MDbmDefine.MIN_SECTOR ;
        if( n <= MDbmDefine.MIN_SECTOR ) {
            n = MDbmDefine.MIN_SECTOR ;
        }
        SECTOR = n ;
        if( logOut.isDebugEnabled() ) {
            logOut.debug( "## MDbmEnv.SECTOR:" + n ) ;
        }
        
        // １セクタファイルで扱える最大数.
        n = MDbmDefine.DEF_ONE_FILE_SECTOR ;
        if( System.getProperty( MDbmDefine.ONE_FILE_SECTOR_PROPERTY ) != null ) {
            try {
                n = Integer.parseInt( System.getProperty( MDbmDefine.ONE_FILE_SECTOR_PROPERTY ) ) ;
                if( n >= MDbmDefine.MAX_ONE_FILE_SECTOR ) {
                    n = MDbmDefine.MAX_ONE_FILE_SECTOR ;
                }
                else if( n <= MDbmDefine.MIN_ONE_FILE_SECTOR ) {
                    n = MDbmDefine.MIN_ONE_FILE_SECTOR ;
                }
            } catch( Exception e ) {
                n = MDbmDefine.DEF_ONE_FILE_SECTOR ;
            }
        }
        n = ( int )( n / MDbmDefine.MIN_ONE_FILE_SECTOR ) * MDbmDefine.MIN_ONE_FILE_SECTOR ;
        if( n <= MDbmDefine.MIN_ONE_FILE_SECTOR ) {
            n = MDbmDefine.MIN_ONE_FILE_SECTOR ;
        }
        ONE_SECTOR_BY_SECTOR_SIZE = n ;
        if( logOut.isDebugEnabled() ) {
            logOut.debug( "## MDbmEnv.ONE_SECTOR_BY_SECTOR_SIZE:" + n ) ;
        }
        
    }
}

